#!/usr/bin/python
# ##########################################################################################
#	parseresults.py  -- parse run from garmin training center
#
#	Date		Author		Reason
#	----		------		------
#	03/27/11	Lou King	Create
#
# ##########################################################################################

# standard
import pdb
import optparse
import csv

# ##########################################################################################
def main():
# ##########################################################################################
    usage = "usage: %prog [options] textresults\n\n"
    usage += "where:\n"
    usage += "  <textresults>\ttext file with results\n"
    usage += "  output is put in .csv file with same name\n"

    parser = optparse.OptionParser(usage=usage)
    parser.add_option("-H", "--hdr", dest="hdr", type='int', help="row number for header", default=None)
    parser.set_defaults()
    (options, args) = parser.parse_args()

    textresults = args.pop(0)
    IN = open(textresults, 'r')
    _OUT = open('{0}.csv'.format(textresults),'w',newline='')
    OUT = csv.writer(_OUT)
    
    hdrqueue = []
    foundhdr = False
    
    if options.hdr:
        rowsuntilhdr = options.hdr
        
    # read each row in the input
    for row in IN:
        # if we haven't found the header yet, it's ok to assume we're not processing data yet either
        if not foundhdr:
            hdrqueue.append(row)
            if not options.hdr:
                if len(hdrqueue) > 2: hdrqueue.pop(0)
            else:
                rowsuntilhdr -= 1
            fields = row.split()
            # are we at the row with the heading =='s?
            if (options.hdr and rowsuntilhdr == 0) or (len(fields)>1 and fields[0][0:2] == '=='):
                foundhdr = True
                # make a list of (start,finish) index tuples for the field data
                boundaries = []
                scanndx = 0
                for field in fields:
                    # find start of next field
                    boundaries.append((scanndx,scanndx+len(field)))
                    scanndx += len(field)
                    while scanndx < len(row) and row[scanndx] != '=': scanndx += 1
                # capture header information from previous row
                headers = []
                for boundary in boundaries:
                    hdrrow = hdrqueue[0]
                    headers.append(hdrrow[boundary[0]:boundary[1]].strip())
                OUT.writerow(headers)
            else:
                continue
                    
        # read data rows, splitting according to boundaries
        fields = []
        for boundary in boundaries:
            fields.append(row[boundary[0]:boundary[1]].strip())
        try:
            # if this is ok, then it's a real data row
            placeoa = int(fields[0])
        except ValueError:
            # else continue reading data
            continue
        
        OUT.writerow(fields)
            
    _OUT.close()
    
# ##########################################################################################
#	__main__
# ##########################################################################################
if __name__ == "__main__":
    main()


#!/usr/bin/python
###########################################################################################
#   runningahead - access methods for runningahead.com
#
#   Date        Author      Reason
#   ----        ------      ------
#   01/08/13    Lou King    Create
#
#   Copyright 2012 Lou King
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
###########################################################################################
'''
runningahead - access methods for runningahead.com
===================================================
'''

# standard
import pdb
import argparse
import os.path
import logging
import json
from tempfile import NamedTemporaryFile

# pypi
import requests
from requests_oauthlib import OAuth2Session
from oauthlib.oauth2 import BackendApplicationClient

# github

# other

# home grown
# from running import *
from loutilities import apikey

# OAuth stuff
auth_url = 'https://www.runningahead.com/oauth2/authorize'
token_url = 'https://api.runningahead.com/oauth2/token'

FIELD = {}
FIELD['workout'] = {
    'eventtype':10,
    'eventsubtype':11,
    'date':12,
    'distance':20,
    'duration':21,  # seconds
    'coursename':22,
    }
KMPERMILE = 1.609344

class accessError(Exception): pass

#----------------------------------------------------------------------
def dist2miles(distance):
#----------------------------------------------------------------------
    '''
    convert distance to miles for distance returned from runningahead
    
    :param dist: distance field from runningahead
    :rtype: float - number of miles represented by the distance field
    '''
    mpermile = KMPERMILE * 1000
    
    unit = distance['unit']
    
    if unit == 'mi':
        distmiles =  distance['value']
    
    elif unit == 'km':
        distmiles = distance['value'] / KMPERMILE
        
    elif unit == 'm':
        distmiles = distance['value'] / mpermile
        
    else:
        raise parameterError('{0}: invalid unit returned for runningahead distance'.format(unit))
    
    return distmiles

#----------------------------------------------------------------------
def dist2meters(distance):
#----------------------------------------------------------------------
    '''
    convert distance to meters for distance returned from runningahead
    
    :param dist: distance field from runningahead
    :rtype: float - number of meters represented by the distance field
    '''
    mpermile = KMPERMILE * 1000
    
    unit = distance['unit']
    
    if unit == 'mi':
        distmeters =  distance['value'] * mpermile
    
    elif unit == 'km':
        distmeters = distance['value'] * 1000.0
        
    elif unit == 'm':
        distmeters = distance['value']
        
    else:
        raise parameterError('{0}: invalid unit returned for runningahead distance'.format(unit))
    
    return distmeters

########################################################################
class RunningAhead():
########################################################################
    '''
    access methods for RunningAhead.com

    :param membercachefilename: name of optional file to cache detailed member data
    :param debug: set to True for debug logging of http requests, default False
    :param key: ra key for oauth, if omitted retrieved from apikey
    :param secret: ra secret for oauth, if omitted retrieved from apikey
    '''

    #----------------------------------------------------------------------
    def __init__(self, membercachefilename=None, debug=False, key=None, secret=None):
    #----------------------------------------------------------------------
        """
        initialize oauth authentication, and load member cache
        """

        # does user want to debug?
        if debug:
            # set up debug logging
            logging.basicConfig() # you need to initialize logging, otherwise you will not see anything from requests
            logging.getLogger().setLevel(logging.DEBUG)
            requests_log = logging.getLogger("requests.packages.urllib3")
            requests_log.setLevel(logging.DEBUG)
            requests_log.propagate = True
        else:
            pass # how to stop?

        # get credentials from configuration if not provided
        if not (key and secret):
            ak = apikey.ApiKey('Lou King','running')
            try:
                key = ak.getkey('ra')
                secret = ak.getkey('rasecret')
            except apikey.unknownKey:
                raise parameterError("'ra' and 'rasecret' keys needs to be configured using apikey")
        
        # Step 3 from http://api.runningahead.com/docs/authentication (using client_credentials, not authorization_code)
        # see http://requests-oauthlib.readthedocs.org/en/latest/oauth2_workflow.html#legacy-application-flow
        client = BackendApplicationClient(client_id=key)
        oauth = OAuth2Session(client=client)
        data = oauth.fetch_token(
            token_url='https://api.runningahead.com/oauth2/token', 
            client_id=key, 
            client_secret=secret,
            include_client_id=True)
        self.client_credentials = data['access_token']

        # set up session for multiple requests
        self.rasession = requests.Session()

        # bring in cache file, if requested
        self.membercache = {}
        self.membercachefilename = membercachefilename
        if self.membercachefilename:
            # only read cache if file exists
            if os.path.isfile(membercachefilename):
                with open(membercachefilename,'r') as membercachefile:
                    # members are stored one per line, in json format
                    for line in membercachefile:
                        member = json.loads(line)
                        self.membercache[member['id']] = member
        # optimization - no write on close if not updated
        self.membercacheupdated = False

    #----------------------------------------------------------------------
    def close(self):
    #----------------------------------------------------------------------
        '''
        close the connection when we're done, and save the cache
        '''
        # done here
        self.rasession.close()

        # save the cache in a temporary file, if requested and it's been updated
        if self.membercachefilename and self.membercacheupdated:
            # get full path for self.membercachefilename to assure cachedir isn't relative
            cachedir = os.path.dirname(os.path.abspath(self.membercachefilename))

            # save temporary file with cache
            with NamedTemporaryFile(mode='w', suffix='.racache', delete=False, dir=cachedir) as tempcache:
                tempmembercachefilename = tempcache.name
                for id in self.membercache:
                    tempcache.write('{}\n'.format(json.dumps(self.membercache[id])))

            # set mode of temp file to be same as current cache file (see https://stackoverflow.com/questions/5337070/how-can-i-get-a-files-permission-mask)
            cachemode = os.stat(self.membercachefilename).st_mode & 0o777
            os.chmod(tempmembercachefilename, cachemode)

            # now overwrite the previous version of the membercachefile with the new membercachefile
            try:
                # atomic operation in Linux
                os.rename(tempmembercachefilename, self.membercachefilename)

            # should only happen under windows
            except OSError:
                os.remove(self.membercachefilename)
                os.rename(tempmembercachefilename, self.membercachefilename)


    #----------------------------------------------------------------------
    def listusers(self):
    #----------------------------------------------------------------------
        """
        return users accessible to this application
        """
        
        # max number of users in user list is 100, so need to loop, collecting
        # BITESIZE users at a time.  These are all added to users list, and final
        # list is returned to the caller
        BITESIZE = 100
        offset = 0
        users = []
        while True:
            data = self._raget('users',
                               self.client_credentials,
                               limit=BITESIZE,
                               offset=offset,
                               )
            if data['numEntries'] == 0:
                break
            theseusers = data['entries']
            
            users += theseusers
            offset += BITESIZE

            # stop iterating if we've reached the end of the data
            if offset >= data['numEntries']:
                break
        
        return users
        
    #----------------------------------------------------------------------
    def listactivitytypes(self,accesstoken):
    #----------------------------------------------------------------------
        """
        return activity types for this user

        :param accesstoken: access_token to use for api call
        """
        
        data = self._raget('logs/me/activity_types',accesstoken)
        activity_types = data['entries']
        return activity_types
        
    #----------------------------------------------------------------------
    def listworkouts(self,accesstoken,begindate=None,enddate=None,getfields=None):
    #----------------------------------------------------------------------
        """
        return run workouts within date range
        
        :param accesstoken: access_token to use for api call
        :param begindate: date in format yyyy-mm-dd
        :param enddate: date in format yyyy-mm-dd
        :param getfields: list of fields to get in response.  See runningahead.FIELD['workout'].keys() for valid codes
        """
        
        if getfields:
            lpfield = []
            for f in getfields:
                lpfield.append(str(FIELD['workout'][f]))
            fields = ','.join(lpfield)
        
        # fill in optional arguments as needed
        optargs = {}
        optargs['activityID'] = 10  # Run
        filters = []
        if begindate: filters.append(['date','ge',begindate])
        if enddate:   filters.append(['date','le',enddate])
        if getfields: optargs['fields']    = fields
        if filters:
            # need to json encode the parameter, so requests doesn't unwravel it, as RA expects the json array of arrays
            optargs['filters'] = json.dumps(filters)
        
        # max number of workouts in workout list is 100, so need to loop, collecting
        # BITESIZE workouts at a time.  These are all added to workouts list, and final
        # list is returned to the caller
        BITESIZE = 100
        offset = 0
        workouts = []
        while True:
            data = self._raget('logs/me/workouts',
                               accesstoken,
                               limit=BITESIZE,
                               offset=offset,
                               **optargs
                               )
            if data['numEntries'] == 0:
                break
            
            theseworkouts = data['entries']
            workouts += theseworkouts
            offset += BITESIZE

            # stop iterating if we've reached the end of the data
            if offset >= data['numEntries']:
                break
        
        # here would be a fine place to operate on an optional filter parameter.
        # only problem with that is every time I do that I make the filter parameter
        # so complex that I can never figure it out myself
        
        return workouts  
        
    #----------------------------------------------------------------------
    def getworkout(self,accesstoken,id):
    #----------------------------------------------------------------------
        """
        return workout for specified id
        
        :param accesstoken: access_token to use for api call
        :param id: id retrieved from listworkouts for desireed workout
        """
        
        data = self._raget('logs/me/workouts/{0}'.format(id),accesstoken)
        workout = data['workout']
        return workout
        
    #----------------------------------------------------------------------
    def getuser(self,accesstoken):
    #----------------------------------------------------------------------
        """
        return workout for specified id
        
        :param accesstoken: access_token to use for api call
        """
        
        data = self._raget('users/me',accesstoken)
        rauser = data['user']
        
        # flatten user structure, as expected by caller
        user = {}
        for raf in rauser:
            if isinstance(rauser[raf], dict):
                for f in rauser[raf]:
                    user[f] = rauser[raf][f]
            else:
                user[raf] = rauser[raf]
                
        return user
        
    #----------------------------------------------------------------------
    def listmemberships(self,club,accesstoken,**filters):
    #----------------------------------------------------------------------
        """
        return list of club memberships
        
        :param club: RA slug name of club
        :param accesstoken: access token for a priviledged viewer for this club
        :param filters: see http://api.runningahead.com/docs/club/list_members for valid filters
        :rtype: list of memberships
        """
        
        # retrieve all the members
        method = 'clubs/{}/members'.format(club)
        data = self._raget(method,accesstoken,**filters)
        memberships = data['entries']
        
        return memberships
        
    #----------------------------------------------------------------------
    def getmember(self,club,id,accesstoken,update=False):
    #----------------------------------------------------------------------
        """
        return list of club members
        
        :param club: RA slug name of club
        :param id: id of member
        :param accesstoken: access token for a priviledged viewer
        :param update: update based on latest information from RA
        :rtype: member record
        """
        
        # do we need to retrieve from RunningAHEAD?
        if update or id not in self.membercache:
            method = 'clubs/{}/members/{}'.format(club,id)
            data = self._raget(method,accesstoken)
            member = data['member']
            self.membercache[id] = member
            self.membercacheupdated = True

        # use member data from cache
        else:
            member = self.membercache[id]
        
        return member
        
    #----------------------------------------------------------------------
    def listmembershiptypes(self,club,accesstoken):
    #----------------------------------------------------------------------
        """
        return list of club membership types
        
        :param club: RA slug name of club
        :param accesstoken: access token for a priviledged viewer for this club
        :rtype: list of memberships
        """
        
        # retrieve all the members
        method = 'clubs/{}/memberships'.format(club)
        data = self._raget(method,accesstoken)
        membershiptypes = data['entries']
        
        return membershiptypes
        
    #----------------------------------------------------------------------
    def _raget(self,method,accesstoken,**payload):
    #----------------------------------------------------------------------
        """
        get method for runningahead access
        
        :param method: runningahead method to call
        :param accesstoken: access_token to use for api call
        :param **payload: parameters for the method
        """
        
        payload['access_token'] = accesstoken
        
        url = 'https://api.runningahead.com/rest/{0}'.format(method)
        r = self.rasession.get(url,params=payload)
        if r.status_code != 200:
            raise accessError('HTTP response code={}, url={}'.format(r.status_code,r.url))

        content = r.json()

        if content['code'] != 0:
            raise accessError('RA response code={}, url={}'.format(content['code'],r.url))
    
        data = content['data']
        return data 
        
#----------------------------------------------------------------------
def main(): 
#----------------------------------------------------------------------
    descr = '''
    unit test for runningahead.py
    '''
    
    parser = argparse.ArgumentParser(description=descr,formatter_class=argparse.RawDescriptionHelpFormatter,
                                     version='{0} {1}'.format('running',version.__version__))
    args = parser.parse_args()

    # this would be a good place for unit tests
    
# ##########################################################################################
#	__main__
# ##########################################################################################
if __name__ == "__main__":
    main()



#!/usr/bin/python
# ##########################################################################################
#	gpx2kml - convert gpx file to kml format
#
#	Date		Author		Reason
#	----		------		------
#	08/29/12	Lou King	Create
#
# ##########################################################################################

# standard
import pdb
import optparse
import datetime
import os.path
from lxml import etree

# pypi
from pykml.factory import KML_ElementMaker as KML
from pykml.factory import GX_ElementMaker as GX

# github
import gpxpy
import gpxpy.geo

# home grown
from loutilities import timeu

METERPMILE = 1609.3439941
t = timeu.asctime('%Y-%m-%dT%H:%M:%SZ')

# ###############################################################################
def main():
# ###############################################################################

    usage = "usage: %prog [options] <gpxfile>\n\n"
    usage += "where:\n"
    usage += "  <gpxfile>\tgpx formatted file"

    parser = optparse.OptionParser(usage=usage)
    parser.add_option("-p", "--points", dest="points", action="store_true", help="specify if points output is desired", default=False)
    parser.add_option("-f", "--flyover", dest="flyover", action="store_true", help="specify if flyover output is desired", default=False)
    parser.add_option("-c", "--color", dest="color", help="track color if not flyover", default='641400FF')
    parser.add_option("-o", "--output", dest="output", help="output file", default=None)
    (options, args) = parser.parse_args()

    # see http://www.zonums.com/gmaps/kml_color/
    colors = {'pink':'64781EF0', 'blue':'64F01E14'}
    
    gpxfile = args.pop(0)
    if options.output == None:
        outfile = os.path.basename(gpxfile) + '.kml'
    else:
        outfile = options.output

    # get input
    _GPX = open(gpxfile,'r')
    gpx = gpxpy.parse(_GPX)

    # create a KML file skeleton
    stylename = "sn_shaded_dot"
    color = colors[options.color]
    sty = KML.Style(
        KML.IconStyle(
            KML.scale(1.2),
            KML.Icon(
                KML.href("http://maps.google.com/mapfiles/kml/shapes/shaded_dot.png")
                ),
            KML.color(colors[options.color]),
            ),
        id=stylename,
        )
    iconstylename = '#sn_shaded_dot'

    doc = KML.Document(
        KML.Name('generated from {0}'.format(gpxfile)),
        KML.open(1),
        )
    doc.append(sty)

    '''
    <TimeStamp>
      <when>1997-07-16T07:30:15Z</when>
    </TimeStamp>
    '''
    # loop through gpx tracks, creating kml placemarks
    times = []
    coords = []
    dists = []
    points = []
    lastpoint = None
    for track in gpx.tracks:
        for segment in track.segments:
            for point in segment.points:
                if not lastpoint:
                    lastpoint = point
                plon = point.longitude
                plat = point.latitude
                pelev = point.elevation
                points.append(point)
                thisdist = gpxpy.geo.distance(lastpoint.latitude, lastpoint.longitude, lastpoint.elevation, plat, plon, pelev)
                lastpoint = point
                dists.append(thisdist)
                ptime = t.dt2asc(point.time)
                times.append(ptime)
                coords.append('{lon},{lat},{alt}'.format(lon = plon,lat = plat,alt = 0))
    
    if options.flyover:
        plm = KML.Placemark()
        doc.append(plm)
        track = GX.track()
        plm.append(track)
        
        for when in times:
            track.append(KML.when(when))
        for coord in coords:
            track.append(KML.coordinates(coord))
            
        '''
        doc.append(KML.Placemark(
            KML.LookAt(
                KML.longitude(plon),
                KML.latitude(plat),
                KML.tilt(45),
                KML.heading(0),  # make this behind the guy
                KML.altitudeMode("relativeToGround"),
                KML.range(50),
                ),
            KML.Point(
                KML.altitudeMode("relativeToGround"),
                ,
                ,
                )
            ))
        '''
        
    elif options.points:
        #for coord in coords:
        #    ls.append(KML.coordinates(coord))
        lasttime = t.asc2epoch(times[0])
        totdist = 0
        for i in range(len(times)):
            thistime = t.asc2epoch(times[i])
            dur = thistime - lasttime
            lasttime = thistime
            totdist += dists[i]
            ex = KML.ExtendedData(
                KML.Data(KML.displayName('time'),KML.value(times[i])),
                KML.Data(KML.displayName('duration'),KML.value(dur)),
                KML.Data(KML.displayName('totdistance'),KML.value(int(round(totdist)))),
                )
            plm = KML.Placemark(
                #KML.name(times[i]),
                KML.name(''),
                KML.styleUrl(iconstylename),
                )
            plm.append(ex)
            plm.append(KML.Point(
                KML.altitudeMode('clampToGround'),
                KML.coordinates(coords[i])))
            doc.append(plm)
    
    else:
        if options.color:
            doc.append(
                KML.Style(
                    KML.LineStyle(
                    KML.color(colors[options.color]),
                    KML.width(5),
                    ),
                id=options.color))
            stylename = '#{0}'.format(options.color)
        
        '''
        <LineString id="ID">
          <!-- specific to LineString -->
          <gx:altitudeOffset>0</gx:altitudeOffset>  <!-- double -->
          <extrude>0</extrude>                      <!-- boolean -->
          <tessellate>0</tessellate>                <!-- boolean -->
          <altitudeMode>clampToGround</altitudeMode> 
              <!-- kml:altitudeModeEnum: clampToGround, relativeToGround, or absolute -->
              <!-- or, substitute gx:altitudeMode: clampToSeaFloor, relativeToSeaFloor -->
          <gx:drawOrder>0</gx:drawOrder>            <!-- integer --> 
          <coordinates>...</coordinates>            <!-- lon,lat[,alt] -->
        </LineString>
        '''
        plm = KML.Placemark(
            KML.name('runtrack'),
            )
        if options.color:
            plm.append(KML.styleUrl(stylename))
        doc.append(plm)
        ls = KML.LineString(
            KML.altitudeMode('clampToGround'),
#            KML.extrude(1),
#            KML.tessellate(1),
            )
        plm.append(ls)
        #for coord in coords:
        #    ls.append(KML.coordinates(coord))
        kcoords = ''
        for coord in coords:
            kcoords += coord + ' \n'
        ls.append(KML.coordinates(kcoords))
        
    _GPX.close()

    kml = KML.kml(doc)
    
    docstr = etree.tostring(kml, pretty_print=True)
    OUT = open(outfile,'w')
    OUT.write(docstr)
    OUT.close()

# ###############################################################################
# ###############################################################################
if __name__ == "__main__":
    main()




#!/usr/bin/python
###########################################################################################
# getweather - get specified weather information from forecast.io
#
#	Date		Author		Reason
#	----		------		------
#       04/10/13        Lou King        Create
#
#   Copyright 2013 Lou King
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
###########################################################################################
'''
getweather - get specified weather information from forecast.io
==============================================================================

'''
# standard
import argparse
import textwrap

# pypi

# github

# other

# home grown
from running.running import version


#----------------------------------------------------------------------
def main():
#----------------------------------------------------------------------

    description = 'Get specified weather information from forecast.io'
    epilog = textwrap.dedent('''\
        paramsfile contains the following
        
            [getweather]
            starttime = <starttime>
            wxpoints = {"dist":[deltat,deltat,...],
                "dist":[deltat,deltat,...],
                ...
                }
        
        where:
            <starttime>\tstarting time for collection, in 'yyyy-mm-dd HH:MM' format (no quotes, local timezone)
            <dist>\tdistance in miles from the first gpx point
            <deltat>\ttime in seconds from <starttime> for weather collection
            
        outfile is a csv file, with a header row, containing the following fields - see https://developer.forecast.io/docs/v2 Data Points for details
        
            exectime\ttime script was executed (Unix time format)
            time\ttime forecast is predicting for (Unix time format)
            lat
            lon
            temperature
            humidity
            dewpoint\tcalculated from temperature, humidity
            windchill\tcalculated from temperature, windSpeed
            heatindex\tcalculated from temperature, humidity
            precipType
            precipProbability
            precipIntensity
            windSpeed
            windBearing
            cloudCover
            summary
            pressure
            visibility
        ''')
    parser = argparse.ArgumentParser(
        prog='getweather.py',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=description,
        epilog=epilog,
        version='{0} {1}'.format('running', version.__version__))
    parser.add_argument('gpxfile',help='gpx file containing course')
    parser.add_argument('paramsfile',help='file containing parameters')
    parser.add_argument('outfile',help='csv file containing output from queries')
    parser.add_argument('-a','--apikey',help='API key to access forecast.io')
    args = parser.parse_args()
    
# ###############################################################################
# ###############################################################################
if __name__ == "__main__":
    main()




==================================================
xmlreader.py:
==================================================
from xml.dom.minidom import parse


class NotTextNodeError:
    pass


def getTextFromNode(node):
    """
    scans through all children of node and gathers the
    text. if node has non-text child-nodes, then
    NotTextNodeError is raised.
    """
    t = ""
    for n in node.childNodes:
	if n.nodeType == n.TEXT_NODE:
	    t += n.nodeValue
	else:
	    raise NotTextNodeError
    return t


def nodeToDic(node):
    """
    nodeToDic() scans through the children of node and makes a
    dictionary from the content.
    three cases are differentiated:
	- if the node contains no other nodes, it is a text-node
    and {nodeName:text} is merged into the dictionary.
	- if the node has the attribute "method" set to "true",
    then it's children will be appended to a list and this
    list is merged to the dictionary in the form: {nodeName:list}.
	- else, nodeToDic() will call itself recursively on
    the nodes children (merging {nodeName:nodeToDic()} to
    the dictionary).
    """
    dic = {} 
    for n in node.childNodes:
	if n.nodeType != n.ELEMENT_NODE:
	    continue
	if n.getAttribute("multiple") == "true":
	    # node with multiple children:
	    # put them in a list
	    l = []
	    for c in n.childNodes:
	        if c.nodeType != n.ELEMENT_NODE:
		    continue
		l.append(nodeToDic(c))
	        dic.update({n.nodeName:l})
	    continue
		
	try:
	    text = getTextFromNode(n)
	except NotTextNodeError:
            # 'normal' node
            dic.update({n.nodeName:nodeToDic(n)})
            continue

        # text node
        dic.update({n.nodeName:text})
	continue
    return dic


def readConfig(filename):
    dom = parse(filename)
    return nodeToDic(dom)





def test():
    dic = readConfig("sample.xml")
    
    print dic["Config"]["Name"]
    print
    for item in dic["Config"]["Items"]:
	print "Item's Name:", item["Name"]
	print "Item's Value:", item["Value"]

test()



==================================================
sample.xml:
==================================================
<?xml version="1.0" encoding="UTF-8"?>

<Config>
    <Name>My Config File</Name>
    
    <Items multiple="true">
	<Item>
	    <Name>First Item</Name>
	    <Value>Value 1</Value>
	</Item>
	<Item>
	    <Name>Second Item</Name>
	    <Value>Value 2</Value>
	</Item>
    </Items>

</Config>



==================================================
output:
==================================================
My Config File

Item's Name: First Item
Item's Value: Value 1
Item's Name: Second Item
Item's Value: Value 2


#!/usr/bin/python
###########################################################################################
#   ultrasignup - access methods for ultrasignup.com
#
#   Date        Author      Reason
#   ----        ------      ------
#   11/23/13    Lou King    Create
#
#   Copyright 2013,2014 Lou King
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
###########################################################################################
'''
ultrasignup - access methods for ultrasignup.com
===================================================
'''

# standard
import argparse
import os.path
import urllib.request, urllib.parse, urllib.error
import unicodedata
import logging
import json
logging.basicConfig(format='%(asctime)s %(levelname)s:%(message)s')

# pypi
import httplib2
# from IPython.core.debugger import Tracer; debug_here = Tracer()

# github

# other

# home grown
from loutilities import timeu
from loutilities import csvu
from loutilities import renderrun as render
from running import accessError, parameterError

# access stuff
ULTRASIGNUP_URL = 'http://ultrasignup.com'
RESULTS_SEARCH = 'service/events.svc/history/{fname}/{lname}'

HTTPTIMEOUT = 10
MPERMILE = 1609.344

tindate  = timeu.asctime('%m/%d/%Y %I:%M:%S %p')
toutdate = timeu.asctime('%Y-%m-%d')

#----------------------------------------------------------------------
def racenameanddist(eventname):
#----------------------------------------------------------------------
    '''
    get race name and distance 
    
    :param eventname: eventname from untrasignup.com
    :rtype: racename, distmiles, distkm
    '''

    # eventname is formatted as <racename> - <dist><units>
    racetext = eventname.strip()
    fields = racetext.split('-')
    
    # include distance in racename, as sometimes it's missing
    racename = racetext
    distfield = fields[-1].strip()
    
    # accumulate distance, and figure out where units field starts
    dist = 0
    startunits = 0
    for digit in distfield:
        if not digit.isdigit():
            break
        dist *= 10
        dist += int(digit)
        startunits += 1
    
    # pull out the units from the distance field, and interpret
    units = distfield[startunits:].strip()
    
    # special cases
    if distfield == 'Marathon':
        distmiles = 26.21875    # true marathon
        distkm = distmiles * (MPERMILE/1000)
        
    elif distfield == '1/2 Marathon':
        distmiles = 13.109375   # true half marathon
        distkm = distmiles * (MPERMILE/1000)
        
    # kilometers
    elif units == 'K':
        distkm = dist 
        distmiles = (dist * 1000) / MPERMILE
    
    # miles
    elif units == 'Miler':
        # some special cases
        if dist == 13:
            distmiles = 13.109375   # true half marathon
        elif dist == 26:
            distmiles = 26.21875    # true marathon
        else:
            distmiles = dist
        distkm = distmiles * (MPERMILE/1000)
    
    else:
        distmiles = None
        distkm = None

    return racename,distmiles,distkm

#----------------------------------------------------------------------
def racenameanddur(eventname):
#----------------------------------------------------------------------
    '''
    get race name and duration 
    
    :param eventname: eventname from untrasignup.com
    :rtype: racename, duration
    '''

    # eventname is formatted as <racename> - <dist><units>
    racetext = eventname.strip()
    fields = racetext.split('-')
    
    # include distance in racename, as sometimes it's missing
    racename = racetext
    durfield = fields[-1].strip()
    
    # accumulate duration, and figure out where units field starts
    dur = 0
    startunits = 0
    for digit in durfield:
        if not digit.isdigit():
            break
        dur *= 10
        dur += int(digit)
        startunits += 1
    
    # pull out the units from the durance field, and interpret
    units = durfield[startunits:]
    
    # hours
    if units == 'hrs':
        duration = dur
    
    else:
        duration = None

    return racename,duration

########################################################################
class UltraSignupResult():
########################################################################
    '''
    holds result from ultrasignup.com
    
    :param ranking: ultra ranking achieved during race
    :param oaplace: overall place
    :param genplace: gender place
    :param age: age on race day
    :param gender: gender
    :param racetime: finishing time h:mm:ss
    :param racedate: date of race yyyy-mm-dd
    :param raceloc: location of race
    :param racename: name of race
    :param distmiles: distance in miles
    :param distkm: distance in kilometers
    '''
    # us_event_attrs are within the json response from ultrasignup.com
    # attrs must be in the same order
    # loop needs to be driven by us_event_attrs because it's shorter than output attrs
    # racename and distance fields are determined from parsing of 'eventname' from ultrasignup.com
    # gender comes from the outer list returned by ultrasignup
    us_event_attrs = 'runner_rank,place,gender_place,age,time,eventdate,city,state'.split(',')    
    attrs = 'ranking,oaplace,genplace,age,racetime,racedate,racecity,racestate,racename,distmiles,distkm,gender'.split(',')
    
    #----------------------------------------------------------------------
    def __init__(self,ranking=None,oaplace=None,genplace=None,age=None,gender=None,racetime=None,racedate=None,raceloc=None,racename=None,distmiles=None,distkm=None):
    #----------------------------------------------------------------------
        self.ranking = ranking
        self.oaplace = oaplace
        self.genplace = genplace
        self.age = age
        self.gender = gender
        self.racetime = racetime
        self.racedate = racedate
        self.raceloc = raceloc
        self.racename = racename
        self.distmiles = distmiles
        self.distkm = distkm

    #----------------------------------------------------------------------
    def __repr__(self):
    #----------------------------------------------------------------------
        reprval = '{}('.format(self.__class__)
        for attr in self.attrs:
            reprval += '{}={},'.format(attr,getattr(self,attr))
        reprval = reprval[:-1]
        reprval += ')'
        return reprval
    
    #----------------------------------------------------------------------
    def set(self,attrvals):
    #----------------------------------------------------------------------
        '''
        set attributes based on list of attr,val pairs
        
        :param attrvals: [(attr,val),...]
        '''
        
        for attr,inval in attrvals:
            val = csvu.str2num(inval)
            
            # special processing for dates
            if attr in ['racedate']:
                val = toutdate.epoch2asc(tindate.asc2epoch(val))
                
            setattr(self,attr,val)

########################################################################
class UltraSignup():
########################################################################
    '''
    access methods for ultrasignup.com
    '''

    #----------------------------------------------------------------------
    def __init__(self,debug=False):
    #----------------------------------------------------------------------
        """
        initialize http 
        """
        # need http object
        self.http = httplib2.Http(timeout=HTTPTIMEOUT)

        # set up logging level
        self.log = logging.getLogger('running.ultrasignup')
        self.setdebug(debug)
        
        # count how many pages have been retrieved
        self.urlcount = 0
        
    #----------------------------------------------------------------------
    def setdebug(self,debugval):
    #----------------------------------------------------------------------
        '''
        set debugging attribute for this class
        
        :param debugval: set to True to enable debugging
        '''
        if not debugval:
            level = logging.INFO
        else:
            level = logging.DEBUG
        self.log.setLevel(level)
        
    #----------------------------------------------------------------------
    def geturlcount(self):
    #----------------------------------------------------------------------
        '''
        each time a url is retrieved, this counter is bumped
        
        :rtype: integer, number of url's retrieved
        '''
        return self.urlcount

    #----------------------------------------------------------------------
    def listresults(self,fname,lname,**filt):
    #----------------------------------------------------------------------
        '''
        return results which match an athlete's name
        
        :param fname: first name of athlete
        :param lname: last name of athlete
        :param **filt: keyword parameters to filter with
        :rtype: list of ultrasignup Race dicts
        '''
        
        # get the data for this athlete
        races = []
        data = self._get(RESULTS_SEARCH.format(
                           fname=urllib.parse.quote(fname),
                           lname=urllib.parse.quote(lname))
                           )
        
        content = json.loads(data)
        
        results = []
        
        # iterate through the rows of the table
        # content contains a list with an entry for each runner of the same name
        for runner in content:
            usresults = runner['Results']
            gender = runner['Gender']
            
            # for each row in the table, grab the row and associate with the attributes
            for usresult in usresults:
                
                # seems like anything other than 1 is not good
                if usresult['status'] != 1: continue
                
                # pull out values from row, in same order as UltraSignupResult.attrs
                vals = []
                for a in UltraSignupResult.us_event_attrs:
                    vals.append(usresult[a])
                
                # zip values into result and parse event name to get name and distances
                result = UltraSignupResult()
                result.set(list(zip(UltraSignupResult.attrs,vals)))
                result.racename,result.distmiles,result.distkm = racenameanddist(usresult['eventname'])
                
                # distmiles == None if this was a timed race.  result.racetime has distance in miles
                if result.distmiles == None:
                    result.racename,duration = racenameanddur(usresult['eventname'])
                    if duration is None: continue   # didn't recognize units
                    
                    result.distmiles = result.racetime
                    result.distkm = result.distmiles * (MPERMILE/1000)
                    # this is in hours so should render correctly
                    result.racetime = render.rendertime(duration*60*60.0,0)
                
                # gender comes from outer runner list
                result.gender = gender
                
                results.append(result)
            
        def _checkfilter(check):
            for key in filt:
                if not hasattr(check,key) or getattr(check,key) != filt[key]:
                    return False
            return True

        results = list(filter(_checkfilter,results))
        return results
        
    #----------------------------------------------------------------------
    def _get(self,method,**params):
    #----------------------------------------------------------------------
        """
        get method for ultrasignup access
        
        :param method: ultrasignup method to call
        :param **params: parameters for the method
        """
        
        body = urllib.parse.urlencode(params)
        url = '{}/{}?{}'.format(ULTRASIGNUP_URL,method,body)
        
        # loop RETRIES times for timeout
        retries = 10
        while retries > 0:
            retries -= 1
            try:
                self.log.debug(url)
                resp,content = self.http.request(url)
                self.urlcount += 1
                break
            except Exception as e:
                if retries == 0:
                    self.log.info('{} requests attempted'.format(self.geturlcount()))
                    self.log.error('http request failure, retries exceeded: {0}'.format(e))
                    raise
                self.log.warning('http request failure: {0}'.format(e))
        
        if resp.status != 200:
            raise accessError('URL response status = {0}'.format(resp.status))
        
        return content 
        
#----------------------------------------------------------------------
def main(): 
#----------------------------------------------------------------------
    descr = '''
    unit test for ultrasignup.py
    '''
    
    parser = argparse.ArgumentParser(description=descr,formatter_class=argparse.RawDescriptionHelpFormatter,
                                     version='{0} {1}'.format('running',version.__version__))
    args = parser.parse_args()

    # this would be a good place for unit tests
    
# ##########################################################################################
#	__main__
# ##########################################################################################
if __name__ == "__main__":
    main()


#!/usr/bin/python
###########################################################################################
#   ra2membersfile - retrieve RunningAHEAD member file to be put into file similar to RA export file
#
#   Date        Author      Reason
#   ----        ------      ------
#   09/17/15    Lou King    Create
#
#   Copyright 2015 Lou King
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
###########################################################################################
'''
ra2membersfile - retrieve RunningAHEAD member file to be put into file similar to RA export file
=========================================================================================================
'''

# standard
import unicodecsv
import argparse
import logging

# pypi

# github

# other

# home grown
from running.running import runningahead

filehdr = ["MemberID","MembershipType","FamilyName","GivenName","MiddleName","Gender","DOB","Email","EmailOptIn","PrimaryMember","RenewalDate","JoinDate","ExpirationDate","Street1","Street2","City","State","PostalCode","Country","Telephone","EntryType"
]
from loutilities import csvwt

# set up logging
logging.basicConfig() # you need to initialize logging, otherwise you will not see anything from requests
logger = logging.getLogger("ra2membersfile")
logger.setLevel(logging.DEBUG)
logger.propagate = True

#----------------------------------------------------------------------
def adddetails(details, memberrecord, debug=False):
#----------------------------------------------------------------------
    '''
    add details of membership to member record

    :param id: member implied
    :param memberrecord: dict to be updated with detailed information
    '''

    memberrecord['FamilyName'] = details['person'].get('familyName',None)
    memberrecord['GivenName'] = details['person'].get('givenName',None)
    memberrecord['MiddleName'] = details['person'].get('middleName',None)
    gender = details['person'].get('gender',None)
    memberrecord['Gender'] = 'Male' if gender == 'm' else 'Female' if gender == 'f' else None
    memberrecord['DOB'] = details['person'].get('birthDate',None)
    memberrecord['Email'] = details['person'].get('email',None)
    memberrecord['Telephone'] = details['person'].get('phone',None)

    # EmailOptIn

    # RenewalDate
    # ExpirationDate may have already been put in record, if so that takes precedence because it is
    # specific to this record while details has latest expiration date for member
    if 'ExpirationDate' not in memberrecord or not memberrecord['ExpirationDate']:
        memberrecord['ExpirationDate'] = details.get('expiration',None)
    memberrecord['JoinDate'] = details.get('join',None)

    memberrecord['Street1'] = details['address'].get('street1',None)
    memberrecord['Street2'] = details['address'].get('street2',None)
    memberrecord['City'] = details['address'].get('city',None)
    memberrecord['State'] = details['address'].get('state',None)
    memberrecord['PostalCode'] = details['address'].get('postalCode',None)
    memberrecord['Country'] = details['address'].get('country',None)

    # EntryType

    if debug:
        last = memberrecord['FamilyName']
        first = memberrecord['GivenName']
        try:
            logger.debug('processing {}, {}'.format(last, first))
        except UnicodeEncodeError:
            # just ignore this error
            pass


#----------------------------------------------------------------------
def ra2members(club, accesstoken, membercachefilename=None, update=False, filename=None, debug=False, key=None, secret=None, **filters):
#----------------------------------------------------------------------
    '''
    retrieve RunningAHEAD members and create a file or list containing
    the member data, similar to export format from RunningAHEAD

    :param club: RunningAHEAD slug for club name
    :param accesstoken: access token for a priviledged viewer for this club
    :param membercachefilename: name of optional file to cache detailed member data
    :param update: update member cache based on latest information from RA
    :param filename: name of file for output. If None, list is returned and file is not created
    :param debug: True turns on requests debug
    :param key: ra key for oauth, if omitted retrieved from apikey
    :param secret: ra secret for oauth, if omitted retrieved from apikey
    :param filters: see http://api.runningahead.com/docs/club/list_members for valid filters
    '''

    # initialize
    ra = runningahead.RunningAhead(membercachefilename=membercachefilename, debug=debug, key=key, secret=secret)
    memberlist = csvwt.wlist()
    members = unicodecsv.DictWriter(memberlist,filehdr)
    members.writeheader()

    # retrieve membership types
    membershiptypes = ra.listmembershiptypes(club,accesstoken)
    mshipxlate = {}
    for membershiptype in membershiptypes:
        mshipxlate[membershiptype['id']] = membershiptype['name']

    # retrieve memberships
    memberships = ra.listmemberships(club,accesstoken,**filters)

    # loop for each membership, saving information for each of the members in the membership
    for membership in memberships:
        member = {'PrimaryMember':'Yes'}
        member['MemberID'] = '@{}'.format(membership['id'])
        member['MembershipType'] = mshipxlate[membership['membershipId']]
        # need to get expiration from top record, else latest expiration is retrieved
        member['ExpirationDate'] = membership.get('expiration',None)
        adddetails(ra.getmember(club, membership['id'], accesstoken, update=update), member, debug)
        members.writerow(member)

        # collect records for secondary members, if there are any
        member['PrimaryMember'] = None
        if 'members' in membership:
            for thismember in membership['members']:
                adddetails(ra.getmember(club, thismember['id'], accesstoken, update=update), member, debug)
                members.writerow(member)

    # clean up
    ra.close()
    
    # write the file if requested
    if filename:
        with open(filename,'w',newline='') as outfile:
            outfile.writelines(memberlist)

    return memberlist

#----------------------------------------------------------------------
def file2members(fname):
#----------------------------------------------------------------------
    '''
    debug function to read file created by ra2members and return memberlist

    :param fname: name of file
    :rtype: list of strings read from file, suitable for input to csv.DictReader
    '''
    with open(fname,'r',newline='') as infile:
        memberlist = infile.readlines()

    return memberlist
    
#----------------------------------------------------------------------
def main(): 
#----------------------------------------------------------------------
    descr = '''
    retrieve members from RunningAHEAD and create a file similar to RA export file
    '''
    
    parser = argparse.ArgumentParser(description=descr,formatter_class=argparse.RawDescriptionHelpFormatter,
                                     version='{0} {1}'.format('running',version.__version__))
    args = parser.parse_args()

    # this would be a good place for unit tests
    
# ##########################################################################################
#   __main__
# ##########################################################################################
if __name__ == "__main__":
    main()


#!/usr/bin/python

# Copyright 2020 Lou King
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# run from directory above setup.py, e.g.,
#    python -m running.setup install sdist bdist_wheel

# home grown
from running import version

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name = "runtilities",
    version = version.__version__,
    packages = find_packages(),
    long_description = long_description,
    long_description_content_type = 'text/markdown',

    scripts = [
        'running/analyzeagegrade.py',
        'running/athlinksresults.py',
        'running/competitor2csv.py',
        'running/parseresults.py',
        'running/parsetcx.py',
        'running/renderclubagstats.py',
        'running/runningaheadresults.py',
        # 'running/strava.py',
        'running/ultrasignupresults.py',
    ],
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',

    # Project uses reStructuredText, so ensure that the docutils get
    # installed or upgraded on the target machine
    # install_requires tailored for runsignup.py
    install_requires = [
        'requests>=2.22.0',
        'requests_oauthlib>=1.3.0',
        'loutilities>=3.3.0',
        'tzlocal>=2.0.0',
        'xlrd>=1.2.0',
        'unicodecsv>=0.14.1',
        'sqlalchemy>=1.3.12',
        # 'matplotlib>=1.1.1', # not available on godaddy
        # 'pykml>=0.1.0',       # lxml not available on godaddy
        # 'lxml>=2.3',          # lxml not available on godaddy
        ],

    entry_points = {
        'console_scripts': [
            'analyzeagegrade = running.analyzeagegrade:main',
            'athlinksresults = running.athlinksresults:main',
            'competitor2csv = running.competitor2csv:main',
            'parseresults = running.parseresults:main',
            'parsetcx = running.parsetcx:main',
            'renderclubagstats = running.renderclubagstats:main',
            'runningaheadresults = running.runningaheadresults:main',
            'ultrasignupresults = running.ultrasignupresults:main',
            # 'updatestravaclubactivitycache = running.strava:updatestravaclubactivitycache'
        ],
    },

    zip_safe = False,

    # metadata for upload to PyPI
    description = 'running related scripts',
    author = 'Lou King',
    author_email = 'lking@pobox.com',
    url = 'http://github.com/louking/running',
    # could also include download_url, etc.
)

